using AutoScan.Models;
using AutoScan.Options;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using System.Net.Http.Json;

namespace AutoScan;

public class ShinobiConnector
{
    private readonly ILogger<ShinobiConnector> _logger;
    private readonly HttpClient _httpClient;
    private readonly ShinobiOptions _options;
    
    public ShinobiConnector(ILogger<ShinobiConnector> logger, HttpClient httpClient, IOptions<ShinobiOptions> options)
    {
        _logger = logger;
        _httpClient = httpClient;
        _options = options.Value;
    }

    public async Task<List<VideoDetail>> FetchMonitorVideosBetween(DateTime from, DateTime to)
    {
        var endpoint = $"{_options.URL}/{_options.APIKey}/videos/{_options.GroupId}/{_options.MonitorId}";
        endpoint += $"?start={from:yyyy-MM-ddTHH:mm:sszzz}&end={to:yyyy-MM-ddTHH:mm:sszzz}";
        
        _logger.LogDebug("Fetching videos details from endpoint: {Endpoint}", endpoint);
        
        //get from the server the response with type VideoDetails
        var response = await _httpClient.GetFromJsonAsync<FetchVideoResponse>(endpoint);
        
        if (response is null)
        {
            _logger.LogWarning("No videos found in the specified range");
            return [];
        }
        
        _logger.LogInformation("Found {Count} videos in the specified range", response.videos.Count);
        
        foreach (var video in response.videos.OrderBy(x => x.time))
        {
            video.end = video.end.ToLocalTime();
            video.time = video.time.ToLocalTime();
            _logger.LogDebug("Video: {Filename} - Time: {time} - Ends: {end}", video.filename, video.time, video.end);
        }

        return response.videos;

    }

    public async Task DownloadMonitorVideo(VideoDetail video, string downloadFolder)
    {
        var endpoint = $"{_options.URL}{video.href}";
        _logger.LogDebug("Fetching video from endpoint: {Endpoint}", endpoint);

        //Video filenames format: "monitorId-2025-02-15T07-45-01.mp4"
        var videoTime = video.time.ToString("yyyy-MM-ddTHH-mm-ss");
        var videoPath = Path.Combine(downloadFolder, $"{video.mid}-{videoTime}.{video.ext}");

        try
        {
            _logger.LogDebug("Downloading video...");
            var videoData = await _httpClient.GetByteArrayAsync(endpoint);
            
            //Write the video to the file
            await File.WriteAllBytesAsync(videoPath, videoData);
            _logger.LogInformation("Video downloaded successfully!");
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "An error occurred while downloading the video {video}", video.filename);
            throw;
        }
    }
    


}